@@ -81,6 +81,14 @@ void device_add_connection(struct btd_device *device, DBusConnection *conn,
 void device_remove_connection(struct btd_device *device, DBusConnection *conn,
 				uint16_t handle);
 gboolean device_has_connection(struct btd_device *device, uint16_t handle);
+void device_disconnect(struct btd_device *device);
+
+typedef void (*disconnect_watch) (struct btd_device *device, void *user_data);
+
+guint device_add_disconnect_watch(struct btd_device *device,
+				disconnect_watch watch, void *user_data,
+				GDestroyNotify destroy);
+void device_remove_disconnect_watch(struct btd_device *device, guint id);
 
 #define BTD_UUIDS(args...) ((const char *[]) { args, NULL } )
 
@@ -93,3 +101,6 @@ struct btd_device_driver {
 
 int btd_register_device_driver(struct btd_device_driver *driver);
 void btd_unregister_device_driver(struct btd_device_driver *driver);
+
+struct btd_device *btd_device_ref(struct btd_device *device);
+void btd_device_unref(struct btd_device *device);
