@@ -808,7 +808,7 @@ int store_record(const gchar *src, const gchar *dst, sdp_record_t *rec)
 	char filename[PATH_MAX + 1], key[28];
 	sdp_buf_t buf;
 	int err, size, i;
-	char *pdata, *str;
+	char *str;
 
 	create_name(filename, PATH_MAX, STORAGEDIR, src, "sdp");
 
@@ -819,7 +819,6 @@ int store_record(const gchar *src, const gchar *dst, sdp_record_t *rec)
 	if (sdp_gen_record_pdu(rec, &buf) < 0)
 		return -1;
 
-	pdata = (char *)buf.data;
 	size = buf.data_size;
 
 	str = g_malloc0(size*2+1);
@@ -847,8 +846,8 @@ sdp_record_t *record_from_string(const gchar *str)
 
 	tmp[2] = 0;
 	for (i = 0; i < size; i++) {
-		 memcpy(tmp, str + (i * 2), 2);
-		 pdata[i] = (uint8_t) strtol(tmp, NULL, 16);
+		memcpy(tmp, str + (i * 2), 2);
+		pdata[i] = (uint8_t) strtol(tmp, NULL, 16);
 	}
 
 	rec = sdp_extract_pdu(pdata, size, &len);
@@ -1068,8 +1067,8 @@ int read_device_id(const gchar *srcaddr, const gchar *dstaddr,
 	if (err) {
 		/* FIXME: We should try EIR data if we have it, too */
 
-		/* If we don't have the data, we don't want to go
-		   through the above search every time. */
+		/* If we don't have the data, we don't want to go through the
+		 * above search every time. */
 		lsource = 0xffff;
 		lvendor = 0x0000;
 		lproduct = 0x0000;
